unit Alien;

interface

uses crt, Colors, Coords, AF_Unit;

type TAlien = object
  Coords : TCoords;
  Colors : TColors;
  AlienChar : char; {The character that will repesent the Alien}
  AddX, AddY : ShortInt;

  {The ammount of time b/t moves for the Alien, and the last time the
   Alien moved respectively}
  DelayTime, LastTime : real;

  constructor Init (fc, bc : TColor; ac : char);
  procedure Draw (show : boolean);
  procedure DieingNoise;
  function GoYet : boolean;
end;

type TAliens = object
  Items : array[1..GM_MAX_ALIEN_COUNT] of TAlien;
  Count : byte;

  procedure NextStage;
  procedure Delete (i : byte);
end;

implementation

constructor TAlien.Init (fc, bc : TColor; ac : char);
begin
  Colors.SetColors (fc, bc);
  AlienChar := ac;
  DelayTime := 0.1;
  LastTime := Timer;
end;

procedure TAlien.Draw (show : boolean);
begin
  {show = true draws the Alien and show = false erases it}
  Coords.Locate;
  Colors.UseColors;
  if show then
    write (AlienChar)
  else
    write (GM_SPACE);
end;

procedure TAlien.DieingNoise;
begin
  sound (400);
  delay (50);
  sound (300);
  delay (50);
  sound (400);
  delay (50);
  NoSound;
end;

function TAlien.GoYet : boolean;
begin
  {This is the delay for the Alien; returns true if enough time has
   elapsed since its last move for it to move again}
  if Timer - LastTime > DelayTime then begin
    GoYet := true;
    LastTime := Timer;
  end else GoYet := false;
end;

{************** TAliens **************}

procedure TAliens.NextStage;
var i : byte;
begin
  Count := GM_MAX_ALIEN_COUNT;

  {Color and AlienChar need to be reinitialized at every stage because
   TAliens.Delete copies one Alien where another was}
  for i := 1 to Count do
    Items[i].Init (i, CL_BLACK, Chr (i + Ord ('0')));
end;

procedure TAliens.Delete (i : byte);
begin
  {Deletes an Alien from Items by copying the Alien after the deleted one
   onto it, then copying the Alien after the copied one onto the copied
   one and so on}
  while i < Count do begin
    Items[i] := Items[i + 1];
    inc (i);
  end;
  dec (Count);
end;

end.