unit Bullets;

interface

uses Colors, Coords, AF_Unit;

type TBullet = object
  Colors : TColors;
  Coords : TCoords;
  BulletChar : char; {Character the represent a bullet}
  {The ammount of time b/t moves and the time of the last move
   respectively}
  DelayTime, LastTime : real;
  AddX, AddY : ShortInt;

  constructor Init;
  procedure Draw (show : boolean);
  function GoYet : boolean;

end;

type TBullets = object
  Items : array[1..GM_MAX_BULLETS] of TBullet;
  Count : byte;

  constructor Init;
  procedure Add (x, y : TCoord; addx, addy : ShortInt);
  procedure Delete (i : byte);
  function OnWhich (c : TCoords) : byte;
end;

implementation

constructor TBullet.Init;
begin
  BulletChar := '.';
  Colors.SetColors (CL_WHITE, CL_BLACK);
end;

procedure TBullet.Draw (show : boolean);
begin
  {show = true draws TBullet; show = false erases it}
  Coords.Locate;
  Colors.UseColors;

  if show then
    write (BulletChar)
  else
    write (GM_SPACE);
end;

function TBullet.GoYet : boolean;
begin
  {returns true if enough time has elapsed for TBullet to move again}
  if timer - LastTime > DelayTime then begin
    GoYet := true;
    LastTime := timer;
  end else GoYet := false;
end;

{************* TBullets *************}

constructor TBullets.Init;
var i : byte;
begin
  Count := 0;

  for i := 1 to GM_MAX_BULLETS do
    Items[i].Init;
end;

procedure TBullets.Add (x, y : TCoord; addx, addy : ShortInt);
begin
  {Adds another TBullet to the Items array if possible}

  if Count >= GM_MAX_BULLETS then exit;

  inc (Count);

  Items[Count].Coords.SetCoords (x, y);
  Items[Count].AddX := addx;
  Items[Count].AddY := addy;
end;

procedure TBullets.Delete (i : byte);
begin
  {Deletes an TBullet from Items by copying the TBullet after the
   deleted one onto it, then copying the TBullet after the copied one
   onto the copied one and so on}
  while i < Count do begin
    Items[i] := Items[i + 1];
    inc (i);
  end;

  dec (Count);
end;

function TBullets.OnWhich (c : TCoords) : byte;
var i : byte;
begin
  {Returns the array index in Items of the TBullet c is on or 0 if c
   in not on a TBullet in Items}
  if Count = 0 then begin
    OnWhich := 0;
    exit;
  end;

  for i := 1 to Count do
    if Items[i].Coords.On (c) then begin
      OnWhich := i;
      exit;
    end;

  OnWhich := 0;
end;

end.