unit Game;

interface

uses crt, Screen, Ship, AF_Unit, Alien, Coords, Colors;

type TGame = object
    Scr : TScreen;
    Ship : TShip;
    Aliens : TAliens;
    OrigTextMode : integer;

    constructor Init;
    destructor GameEnd;
    procedure Over;
    procedure Won;
    procedure Credits;
    procedure NextStage;
    procedure MainLoop;
    procedure GetRndCoords (var c : TCoords; AlienIndex : byte);
    function OnAnyAlien (c : TCoords; AlienIndex : byte) : boolean;
    function OnWhatAlien (c : TCoords) : byte;
    procedure MoveShip;
    procedure MoveBullets;
    function MoveBullet (i : byte) : boolean;
    function BulletHit (i : byte) : boolean;
    procedure FigureAlienAddXY (i : byte);
    procedure MoveAlien (i : byte);
    procedure MoveAliens;
    procedure KillAlien (i : byte);
    procedure AliensNextStage;
end;

implementation

constructor TGame.Init;
begin
  {Used to restore the text mode to what it was before the program
   ran}
  OrigTextMode := LastMode;
  TextMode (CO80);
  ChangeCursor (false);
  Randomize;

  Scr.Init;
  Ship.Init;
  AliensNextStage;

  MainLoop;
end;

destructor TGame.GameEnd;
begin
  Credits;
  TextMode (OrigTextMode);
  halt;
end;

procedure TGame.Over;
begin
  {called when Aliens beat Ship}
  sleep (1);

  SetFBColors (CL_WHITE, CL_BLACK);
  ClrScr;
  CenterX ('Game Over', SC_VERT_CENTER);
  sleep (1);

  GameEnd;
end;

procedure TGame.Won;
begin
  {called when the Ship beats the Aliens}
  sleep (1);

  SetFBColors (CL_WHITE, CL_BLACK);
  ClrScr;
  CenterX ('Congradulations!!! You Won!!!', SC_VERT_CENTER);
  sleep (1);

  GameEnd;
end;

procedure TGame.Credits;
begin
  ClrScr;

  SetFBColors (CL_WHITE, CL_BLACK);
  CenterX ('Alien Force', SC_VERT_CENTER - 2);
  sleep (1);
  CenterX ('Game by: Jimmy Reumuth', SC_VERT_CENTER);
  sleep (1);
  CenterX ('December 31, 1998', SC_VERT_CENTER + 2);

  repeat until KeyPressed;
end;

procedure TGame.NextStage;
begin
  if Scr.Stage = GM_MAX_STAGE then begin
    Won;
    exit;
  end;

  Scr.NextStage;
  Ship.NextStage;
  AliensNextStage;
end;

procedure TGame.MainLoop;
begin
  repeat
    case GetKey of
      VK_UP : begin
        Ship.NAddX := 0;
        Ship.NAddY := -1;
      end;

      VK_DOWN : begin
        Ship.NAddX := 0;
        Ship.NAddY := 1;
      end;

      VK_LEFT : begin
        Ship.NAddX := -1;
        Ship.NAddY := 0;
      end;

      VK_RIGHT : begin
        Ship.NAddX := 1;
        Ship.NAddY := 0;
      end;

      VK_SPACE :
        {Since Bullet's AddX & Y will = Ship's AddX & Y, the Bullet
         won't move if AddX & Y are both 0; in TGame.MoveBullet the
         cooridnates of Bullet are not checked until after it is
         moved, so it cannot start off on a wall}
        if ((Ship.AddX <> 0) or (Ship.AddY <> 0))
        and (not Scr.OnWall (Ship.Coords.x + Ship.AddX, Ship.Coords.y + Ship.AddY)) then
          Ship.Fire;

      VK_ESC : GameEnd;
    end; {case}
    MoveShip;
    MoveBullets;
    MoveAliens;
    {Change the stage if all Aliens have been killed}
    if Aliens.Count = 0 then NextStage;
  until false;
end;

procedure TGame.GetRndCoords (var c : TCoords; AlienIndex : byte);
begin
  {Get a random number above GM_LEFT and below GM_RIGHT for c
   until its not on a Wall, Ship, or Alien; since this function is
   used by Aliens, it will have to let the Alien be on itself;
   AlienIndex is the index of the Alien calling this function (and
   0 if not called by an Alien), and OnAnyAlien will return false if
   c is only on the Alien with index of AlienIndex}
  repeat
    c.x := Random (GM_RIGHT - GM_LEFT - 1) + GM_LEFT + 1;
    c.y := Random (GM_BOT - GM_TOP - 1) + GM_TOP + 1;
  until
    (not Scr.OnWall (c.x, c.y))
    and (not Ship.Coords.On (c))
    and (not OnAnyAlien (c, AlienIndex))
end;

function TGame.OnAnyAlien (c : TCoords; AlienIndex : byte) : boolean;
var i : byte;
begin
  {Retrusn true only if c is on an Alien that does not have an index
   of AlienIndex in TAliens.Items}
  for i := 1 to Aliens.Count do
    if Aliens.Items[i].Coords.On (c) then begin
      if i = AlienIndex then continue;
      OnAnyAlien := true;
      exit;
    end;

  OnAnyAlien := false;
end;

function TGame.OnWhatAlien (c : TCoords) : byte;
var i : byte;
begin
  {Returns the index of the Alien c is on or 0 if c is not on an
   Alien}
  for i := 1 to Aliens.Count do
    if Aliens.Items[i].Coords.On (c) then begin
      OnWhatAlien := i;
      exit;
    end;

  OnWhatAlien := 0;
end;

procedure TGame.MoveShip;
begin
  if not Ship.GoYet then exit;

  {NAddX & Y are changed when the the player presses up, down, left,
   or right, and they are assigned (through TShip.NextToAdd) to
   AddX & Y when Ship.Coords.x + Ship.AddX, Ship.Coords.y + Ship.AddY
   are a valid position (not on a wall); this allows the user to
   press right when going down and keep going down and not turn right
   until he can rather than just stopping if he can't turn right}
  if not Scr.OnWall (Ship.Coords.x + Ship.NAddX, Ship.Coords.y + Ship.NAddY) then
    Ship.NextToAdd;

  {Ship cannot move into a wall}
  if Scr.OnWall (Ship.Coords.x + Ship.AddX, Ship.Coords.y + Ship.AddY) then exit;

  Ship.Draw (false);
  Ship.Coords.SetCoords (Ship.Coords.x + Ship.AddX, Ship.Coords.y + Ship.AddY);
  Ship.Draw (true);

  {When TransCount is positive the Ship is blinking; actual blinking
   is handled in TShip.Draw; when Ship is killed or game starts,
   TransCount is set to GM_MAX_TRANS_COUNT, then decreased (while
   Ship blinks) until TransCount = 0}
  if Ship.TransCount > 0 then dec (Ship.TransCount);
end;

procedure TGame.MoveBullets;
var i : byte;
begin
  i := 1;
  {while is used instead of for because a Bullet may be delete if
   it hits something which would decrease Count; if the Bullet
   doesn't hit something, inc (i) happens and the next Bullet is
   moved; if it does hit something, every Bullet in Bullets.Items is
   moved down so the next Bullet to be moved has the same index of the
   Bullet that just got hit}
  while i <= Ship.Bullets.Count do
    if Ship.Bullets.Items[i].GoYet then
      if MoveBullet (i) then inc (i);
end;

function TGame.MoveBullet (i : byte) : boolean;
begin
  {Retrusn true only if the bullet does not hit something which means
   if moved}
  with Ship.Bullets.Items[i] do begin
    Draw (false);
    Coords.SetCoords (Coords.x + AddX, Coords.y + AddY);
    if BulletHit (i) then
      MoveBullet := false {not BulletHit}
    else begin
      Draw (true);
      MoveBullet := true; {not BulletHit}
    end;
  end;{with}
end;

function TGame.BulletHit (i : byte) : boolean;
var i2 : byte;
begin
  {Returns true only if the Bullet hits a wall or Alien}
  with Ship.Bullets.Items[i] do begin
     if Scr.OnWall (Coords.x, Coords.y) then begin
       Ship.Bullets.Delete (i);
       BulletHit := true;
       exit;
     end;

     i2 := OnWhatAlien (Coords);
     if i2 > 0 then begin
       Ship.Bullets.Delete (i);
       KillAlien (i2);
       BulletHit := true;
       exit;
     end;
  end; {with}
  BulletHit := false;
end;

procedure TGame.FigureAlienAddXY (i : byte);
begin
  {This procedure decides what an Alien's AddX & Y values will be}

  with Aliens.Items[i] do begin
    {If the horizontal distance b/t the Alien and Ship is greater
     than the vertical, then the Alien checks to move in that
     direction first}
    if abs (Coords.x - Ship.Coords.x) > abs (Coords.y - Ship.Coords.y) then begin
      {go left is Ship is left of Alien}
      if Coords.x > Ship.Coords.x then begin
        AddX := -1;
        AddY := 0;
        if not Scr.OnWall (Coords.x + AddX, Coords.y) then exit;
      end else if Coords.x < Ship.Coords.x then begin
      {go right is Ship is right of Alien}
        AddX := 1;
        AddY := 0;
        if not Scr.OnWall (Coords.x + AddX, Coords.y) then exit;
      end;

      {go up or down if Ship is neither left or right of Alien}
      if Coords.y > Ship.Coords.y then begin
        AddX := 0;
        AddY := -1;
        if not Scr.OnWall (Coords.x + AddX, Coords.y + AddY) then exit;
      end else if Coords.y < Ship.Coords.y then begin
        AddX := 0;
        AddY := 1;
        if not Scr.OnWall (Coords.x + AddX, Coords.y + AddY) then exit;
      end;
    end else begin {if abs...}
    {If the vertical distance b/t the Alien and Ship is greater
     than the horizontal, then the Alien checks to move in that
     direction first}
      if Coords.y > Ship.Coords.y then begin
      {go up if Ship is above alien}
        AddX := 0;
        AddY := -1;
        if not Scr.OnWall (Coords.x + AddX, Coords.y + AddY) then exit;
      end else if Coords.y < Ship.Coords.y then begin
      {go down if Ship is below Alien}
        AddX := 0;
        AddY := 1;
        if not Scr.OnWall (Coords.x + AddX, Coords.y + AddY) then exit;
      end;

      {go left or right if Ship is vertically the same as Alien}
      if Coords.x > Ship.Coords.x then begin
        AddX := -1;
        AddY := 0;
        if not Scr.OnWall (Coords.x + AddX, Coords.y + AddY) then exit;
      end else if Coords.x < Ship.Coords.x then begin
        AddX := 1;
        AddY := 0;
        if not Scr.OnWall (Coords.x + AddX, Coords.y + AddY) then exit;
      end;
    end; {if abs...}
    {don't move if Alien cannot}
    AddX := 0;
    AddY := 0;
  end; {with...}
end;

procedure TGame.MoveAlien (i : byte);
var i2 : byte;
begin
  with Aliens.Items[i] do begin
    if not GoYet then exit;

    {erases the Alien, moves it, and redraws it}
    Draw (false);
    FigureAlienAddXY (i);
    Coords.SetCoords (Coords.x + AddX, Coords.y + AddY);
    Draw (true);

    {See if the Alien is on a Bullet}
    i2 := Ship.Bullets.OnWhich (Coords);
    if i2 > 0 then begin
      KillAlien (i);
      Ship.Bullets.Delete (i2);
      exit;
    end;

    {See if the Alien ran into the Ship}
    if (CordComp (Coords, Ship.Coords))
    and (Ship.TransCount = 0) then begin
      {KillAlien (i);}
      if Ship.Kill = skGameOver then Over;
    end;
  end;
end;

procedure TGame.MoveAliens;
var i : byte;
begin
  i := 1;
  {A while loop is here for the exact same reason it is in
   MoveBullets}
  while i <= Aliens.Count do begin
    MoveAlien (i);
    inc (i);
  end;
end;

procedure TGame.KillAlien (i : byte);
begin
  {This procedure is called when Alien i is killed}

  {increase the number of Aliens killed since Ship's last death}
  with Ship do begin
    if KillNum < SP_MAX_SHOT_POINTS then inc (KillNum);
    ChangeScore (KillNum);
  end;

  with Aliens do begin
    Items[i].Draw (false);
    Items[i].DieingNoise;
    Delete (i);
  end;
end;

procedure TGame.AliensNextStage;
var i : byte;
begin
  with Aliens do begin
    NextStage;

    for i := 1 to Count do begin
      GetRndCoords (Items[i].Coords, i);
      Items[i].Draw (true);
    end;
  end; {with...}
end;

end.