unit Screen;

interface

uses crt, Colors, AF_Unit;

type
  TScreen = object
    Screen : array [GM_LEFT..GM_RIGHT, GM_TOP..GM_BOT] of char;
    Colors : TColors;
    Stage : byte;

    constructor Init;
    procedure Draw;
    function OnWall (x, y : word) : boolean;
    procedure NextStage;
  end;

implementation

constructor TScreen.Init;
var x, y : byte;
begin
  {Sets initial color to CL_BLACK (which = 0) because NextStage which will
   inc the foreground color will be called at the end of this constructor}
  Colors.SetColors (CL_BLACK, CL_BLACK);

  {The for loops set borders of walls with every other square inside walls
   and every thing else spaces for the screen array}
  for x := GM_LEFT to GM_RIGHT do begin
    Screen[x, GM_TOP] := GM_WALL;
    Screen[x, GM_BOT] := GM_WALL;
  end;

  for y := GM_TOP to GM_BOT do begin
    Screen[GM_LEFT, y] := GM_WALL;
    Screen[GM_RIGHT, y] := GM_WALL;
  end;

  for x := GM_LEFT + 1 to GM_RIGHT - 1 do begin
    Screen[x, GM_TOP + 1] := GM_SPACE;
    Screen[x, GM_BOT - 1] := GM_SPACE;
  end;

  for y := GM_TOP + 1 to GM_BOT - 1 do begin
    Screen[GM_LEFT + 1, y] := GM_SPACE;
    Screen[GM_RIGHT - 1, y] := GM_SPACE;
  end;

  for x := GM_LEFT + 2 to GM_RIGHT - 2 do
    for y := GM_TOP + 2 to GM_BOT - 2 do
      if (x mod 2 = 0) and (y mod 2 = 0) then
        Screen[x, y] := GM_WALL
      else
        Screen[x, y] := GM_SPACE;

  NextStage;
end;

procedure TScreen.Draw;
var x, y : byte;
begin
  {This procedure draws the screen and writes out the stage number in the
   upper right hand corner of the screen}
  Colors.UseColors;
  ClrScr;

  for x := GM_LEFT to GM_RIGHT do
    for y := GM_TOP to GM_BOT do begin
      gotoxy (x, y);
      write (Screen[x, y]);
    end;

  gotoxy (GM_RIGHT - 10, 1);
  write ('Stage = ', Stage);
end;

function TScreen.OnWall (x, y : word) : boolean;
begin
  {Returns true if the coorninates are on the wall and false otherwise}
  OnWall := Screen[x, y] = GM_WALL;
end;

procedure TScreen.NextStage;
begin
  inc (Stage);
  inc (Colors.ForeColor);
  Draw;
end;

end.