unit Ship;

interface

uses crt, Coords, Colors, AF_Unit, Bullets;

const
  {The ammount of moves between blinks}
  SP_TRANS_LEN = 10; {SP --> Ship}
  {The ammount of moves for which TShip is transparent}
  SP_MAX_TRANS_COUNT = 30;
  {The ammoun of points before TShip is give a free life}
  SP_1UP = 200;
  {The maximum number of points that can be earned from shooting an
   Alien}
  SP_MAX_SHOT_POINTS = 10;

type TShipKill = (skGameOver, skContinue);

type TShip = object
  Coords : TCoords;
  ShipChar : char;
  Colors : TColors;
  AddX, AddY, NAddX, NAddY : ShortInt; {NAddX --> Next AddX}
  {The ammount of time b/t moves, and the time of the last move
   respectively}
  DelayTime, LastTime : real;
  Bullets : TBullets;
  KillNum : byte; {Aliens killed since last death}
  Lives : byte;
  LastScore, Score : word;
  TransCount : byte;

  constructor Init;
  procedure NextStage;
  function Blink : boolean;
  procedure Draw (show : boolean);
  procedure NextToAdd;
  function GoYet : boolean;
  procedure Fire;
  procedure FiringNoise;
  function Kill : TShipKill;
  procedure DieingNoise;
  procedure _1UpNoise;
  procedure ChangeScore (ds : byte);
  procedure ShowScore;
  procedure ChangeLives (dl : ShortInt);
  procedure ShowLives;
end;

implementation

constructor TShip.Init;
begin
  ShipChar := 'S';
  Lives := 3;
  Score := 0;
  LastScore := Score;
  KillNum := 0;
  NextStage;
end;

procedure TShip.NextStage;
begin
  Coords.SetCoords (GM_LEFT + 1, GM_TOP + 1);
  AddX := 0; AddY := 0;
  NAddX := 0; NAddY := 0;
  Colors.SetColors (CL_YELLOW, CL_BLACK);
  ShowLives;
  ShowScore;
  Bullets.Init;
  TransCount := SP_MAX_TRANS_COUNT;
  DelayTime := 0.05;
  LastTime := Timer;
  Draw (true);
end;

function TShip.Blink : boolean;
begin
  {Returns true if the TShip should be blinking another color; true
   half the the time when the TShip is transparent}
  Blink := (TransCount mod SP_TRANS_LEN) >= (SP_TRANS_LEN div 2);
end;

procedure TShip.Draw (show : boolean);
begin
  {show = true and TShip is drawn; show = false and TShip is erased}
  Coords.Locate;
  Colors.UseColors;

  {When blinking the blink color is the lighter version of TShip's
   color if it's a dark color, and the darker version if it's a light
   color}
  if Blink then
    TextColor ((Colors.ForeColor + LIGHT_TO_DARK) mod (2 * LIGHT_TO_DARK));

  if show then
    write (ShipChar)
  else
    write (GM_SPACE);
end;

procedure TShip.NextToAdd;
begin
  AddX := NAddX;
  AddY := NAddY;
end;

function TShip.GoYet : boolean;
begin
  {returns true if enough time has elapsed for TShip to move again}
  if Timer - LastTime > DelayTime then begin
    GoYet := true;
    LastTime := Timer;
  end else GoYet := false;
end;

procedure TShip.Fire;
begin
  Bullets.Add (Coords.x + AddX, Coords.y + AddY, AddX, AddY);
  FiringNoise;
end;

procedure TShip.FiringNoise;
begin
  Sound (100);
  delay (30);
  NoSound;
end;

function TShip.Kill : TShipKill;
begin
  {function for when TShip gets killed; returns skGameOver when lives
   are 0 and game should end and skContineu when game is not over and
   should continue}
  ChangeLives (-1);
  if Lives = 0 then
    Kill := skGameOver
  else begin
    TransCount := SP_MAX_TRANS_COUNT; {make transparent}
    KillNum := 0; {0 Aliens killed since last death (now)}
    Kill := skContinue;
  end;
end;

procedure TShip.DieingNoise;
begin
  sound (200);
  delay (40);
  sound (190);
  delay (40);
  sound (180);
  delay (40);
  sound (170);
  delay (40);
  sound (160);
  delay (40);
  NoSound;
end;

procedure TShip._1UpNoise;
begin
  sound (600);
  delay (100);
  sound (800);
  delay (100);
  sound (1000);
  delay (100);
  sound (1200);
  delay (100);
  sound (900);
  delay (100);
  sound (600);
  delay (100);

  NoSound;
end;

procedure TShip.ChangeScore (ds : byte); {ds --> delta (change in) score}
begin
  {Increases the score by ds; called when a Alien is killed and KillNum
   will be passed to ds}
  Score := Score + ds;
  ShowScore;

  {Gives a free life if TShip has earned enough points}
  if (Score div SP_1UP) > (LastScore div SP_1UP) then ChangeLives (1);
  LastScore := Score;
end;

procedure TShip.ShowScore;
begin
  Colors.UseColors;
  gotoxy (GM_LEFT, 1);
  write ('Score = ', Score, ';');
end;

procedure TShip.ChangeLives (dl : ShortInt);
begin
  {dl --> delta (Change In) lives; changes the ammount of lives by dl}
  Lives := Lives + dl;
  ShowLives;
  if dl < 0 then
    DieingNoise
  else if dl > 0 then
    _1UpNoise;
end;

procedure TShip.ShowLives;
begin
  Colors.UseColors;
  gotoxy (GM_LEFT + 13, 1);
  write ('Lives = ', Lives);
end;

end.